use anyhow::Result;
use cpal::traits::{DeviceTrait, HostTrait, StreamTrait};
use cpal::{Device, Stream, StreamConfig, SampleFormat, SampleRate};
use std::sync::{Arc, Mutex};
use std::time::{Duration, Instant};
use symphonia::core::audio::Signal;
use symphonia::core::codecs::{DecoderOptions};
use symphonia::core::formats::{FormatOptions};
use symphonia::core::io::MediaSourceStream;
use symphonia::core::meta::MetadataOptions;
use symphonia::core::probe::Hint;
use tracing::{error, warn};

pub struct AsioPlayer {
    device: Device,
    stream: Option<Stream>,
    forced_sample_rate: Option<u32>,
    stream_buffer: Option<Arc<Mutex<StreamBuffer>>>,
    duration_ms: Option<u64>,
    start_time: Option<Instant>,
    paused_position: Option<f32>,
    volume: f32,
}

struct StreamBuffer {
    samples: Vec<f32>,
    position: usize,
    sample_rate: u32,
    channels: u16,
    volume: f32,
    paused: bool,
}

impl AsioPlayer {
    pub fn new() -> Result<Self> {
        let host = cpal::host_from_id(cpal::HostId::Asio)?;
        let mut devices = host.output_devices()?;
        let device = devices.next()
            .ok_or_else(|| anyhow::anyhow!("No ASIO output device found"))?;
        //info!("Using ASIO device: {}", device.name()?);

        Ok(Self {
            device,
            stream: None,
            forced_sample_rate: None,
            stream_buffer: None,
            duration_ms: None,
            start_time: None,
            paused_position: None,
            volume: 1.0,
        })
    }
    
    pub fn set_sample_rate(&mut self, sample_rate: u32) -> Result<()> {
        // Validate that the device supports this sample rate
        let supported_configs: Vec<_> = self.device.supported_output_configs()?.collect();
        
        let mut supported = false;
        for config in &supported_configs {
            if config.min_sample_rate().0 <= sample_rate && config.max_sample_rate().0 >= sample_rate {
                supported = true;
                break;
            }
        }
        
        if !supported {
            return Err(anyhow::anyhow!(
                "Sample rate {} Hz is not supported by this device. Supported ranges: {:?}",
                sample_rate,
                supported_configs.iter()
                    .map(|c| format!("{}-{} Hz", c.min_sample_rate().0, c.max_sample_rate().0))
                    .collect::<Vec<_>>()
            ));
        }
        
        self.forced_sample_rate = Some(sample_rate);
        Ok(())
    }
    
    pub fn clear_sample_rate(&mut self) {
        self.forced_sample_rate = None;
        //info!("Sample rate forcing cleared - will use automatic selection");
    }
    
    pub async fn play_url(&mut self, url: &str) -> Result<()> {
        //info!("Starting playback of: {}", url);
        
        // Stop any existing playback
        self.stop()?;
        
        // Download and decode the audio
        let (samples, original_sample_rate, channels, duration_ms) = self.download_and_decode(url).await?;
        
        self.setup_stream(samples, original_sample_rate, channels)?;
        
        self.duration_ms = Some(duration_ms);
        
        // Start playing
        if let Some(stream) = &self.stream {
            stream.play()?;
            self.start_time = Some(Instant::now());
            self.paused_position = None;
            //info!("Playback started");
        }
        
        Ok(())
    }
    
    pub fn pause(&mut self) -> Result<()> {
        if let Some(stream) = &self.stream {
            stream.pause()?;
            
            // Calculate current position when pausing
            if let Some(start_time) = self.start_time {
                let elapsed = start_time.elapsed().as_secs_f32();
                let duration_secs = self.duration_ms.unwrap_or(0) as f32 / 1000.0;
                self.paused_position = Some((elapsed / duration_secs).min(1.0));
            }
            
            // Set paused flag in buffer
            if let Some(buffer) = &self.stream_buffer {
                let mut buffer = buffer.lock().unwrap();
                buffer.paused = true;
            }
            
            //info!("Playback paused");
            Ok(())
        } else {
            Err(anyhow::anyhow!("No active stream to pause"))
        }
    }
    
    pub fn resume(&mut self) -> Result<()> {
        if let Some(stream) = &self.stream {
            // Update start time based on paused position
            if let Some(paused_pos) = self.paused_position {
                let duration_secs = self.duration_ms.unwrap_or(0) as f32 / 1000.0;
                let elapsed_secs = paused_pos * duration_secs;
                self.start_time = Some(Instant::now() - Duration::from_secs_f32(elapsed_secs));
            }
            
            // Clear paused flag in buffer
            if let Some(buffer) = &self.stream_buffer {
                let mut buffer = buffer.lock().unwrap();
                buffer.paused = false;
            }
            
            stream.play()?;
            //info!("Playback resumed");
            Ok(())
        } else {
            Err(anyhow::anyhow!("No active stream to resume"))
        }
    }
    
    pub fn stop(&mut self) -> Result<()> {
        if let Some(stream) = &self.stream {
            // First, clear the buffer to prevent noise
            if let Some(buffer) = &self.stream_buffer {
                let mut buffer = buffer.lock().unwrap();
                buffer.paused = true;  // Stop audio immediately
                buffer.position = 0;   // Reset position
                buffer.samples.clear(); // Clear the buffer completely
            }
            
            std::thread::sleep(std::time::Duration::from_millis(50));
            
            stream.pause()?;
            
            self.stream = None;
            self.stream_buffer = None;
            self.start_time = None;
            self.paused_position = None;
            
            //info!("Playback stopped and buffer cleared");
            Ok(())
        } else {
            // Not an error if already stopped
            Ok(())
        }
    }
    
    pub fn seek(&mut self, position: f32) -> Result<()> {
        // position is 0.0 - 1.0 (percentage of track)
        let position = position.max(0.0).min(1.0);
        
        if let Some(buffer) = &self.stream_buffer {
            let mut buffer = buffer.lock().unwrap();
            let total_frames = buffer.samples.len() / buffer.channels as usize;
            let new_frame = (total_frames as f32 * position) as usize;
            buffer.position = new_frame * buffer.channels as usize;
            
            // Update start time to reflect new position
            if let Some(duration_ms) = self.duration_ms {
                let duration_secs = duration_ms as f32 / 1000.0;
                let elapsed_secs = position * duration_secs;
                self.start_time = Some(Instant::now() - Duration::from_secs_f32(elapsed_secs));
            }
            
            if let Some(_stream) = &self.stream {
                if buffer.paused {
                    self.paused_position = Some(position);
                }
            }
            
            //info!("Seeked to position: {:.1}%", position * 100.0);
            Ok(())
        } else {
            Err(anyhow::anyhow!("No active stream to seek"))
        }
    }
    
    pub fn set_volume(&mut self, volume: f32) -> Result<()> {
        let volume = volume.max(0.0).min(1.0);
        self.volume = volume;
        
        if let Some(buffer) = &self.stream_buffer {
            let mut buffer = buffer.lock().unwrap();
            buffer.volume = volume;
            //info!("Volume set to: {:.0}%", volume * 100.0);
        }
        
        Ok(())
    }
    
    pub fn get_position(&self) -> Option<f32> {
        if let (Some(start_time), Some(duration_ms)) = (self.start_time, self.duration_ms) {
            if let Some(paused_pos) = self.paused_position {
                return Some(paused_pos);
            }
            
            let elapsed = start_time.elapsed().as_millis() as f32;
            let duration = duration_ms as f32;
            Some((elapsed / duration).min(1.0))
        } else {
            None
        }
    }
    
    async fn download_and_decode(&self, url: &str) -> Result<(Vec<f32>, u32, u16, u64)> {
        //debug!("Downloading audio from: {}", url);
        
        let response = reqwest::get(url).await?;
        let bytes = response.bytes().await?;
        
        //debug!("Downloaded {} bytes, decoding...", bytes.len());
        
        // Create media source
        let cursor = std::io::Cursor::new(bytes);
        let media_source = Box::new(cursor);
        let mss = MediaSourceStream::new(media_source, Default::default());
        
        // Probe the format
        let hint = Hint::new();
        let format_opts = FormatOptions::default();
        let metadata_opts = MetadataOptions::default();
        
        let probed = symphonia::default::get_probe()
            .format(&hint, mss, &format_opts, &metadata_opts)?;
            
        let mut format_reader = probed.format;
        let track = format_reader.tracks()
            .iter()
            .find(|t| t.codec_params.codec != symphonia::core::codecs::CODEC_TYPE_NULL)
            .ok_or_else(|| anyhow::anyhow!("No audio track found"))?;
            
        let track_id = track.id;
        let sample_rate = track.codec_params.sample_rate.unwrap_or(44100);
        let channels = track.codec_params.channels.map(|c| c.count()).unwrap_or(2) as u16;
        
        // Create decoder
        let decoder_opts = DecoderOptions::default();
        let mut decoder = symphonia::default::get_codecs()
            .make(&track.codec_params, &decoder_opts)?;
            
        //info!("Decoding: {} channels, {} Hz", channels, sample_rate);
        
        let mut samples = Vec::new();
        
        // Decode packets
        loop {
            match format_reader.next_packet() {
                Ok(packet) => {
                    if packet.track_id() != track_id {
                        continue;
                    }
                    
                    match decoder.decode(&packet) {
                        Ok(decoded) => {
                            // Convert samples based on the actual format
                            match decoded {
                                symphonia::core::audio::AudioBufferRef::F32(buf) => {
                                    let spec = *buf.spec();
                                    if spec.channels.count() == 1 {
                                        // Mono
                                        let chan = buf.chan(0);
                                        samples.extend_from_slice(chan);
                                    } else if spec.channels.count() == 2 {
                                        // Stereo - interleave L/R
                                        let left = buf.chan(0);
                                        let right = buf.chan(1);
                                        
                                        for i in 0..left.len() {
                                            samples.push(left[i]);
                                            samples.push(right[i]);
                                        }
                                    }
                                }
                                symphonia::core::audio::AudioBufferRef::S32(buf) => {
                                    let spec = *buf.spec();
                                    if spec.channels.count() == 1 {
                                        // Mono - convert i32 to f32
                                        let chan = buf.chan(0);
                                        for &sample in chan {
                                            samples.push(sample as f32 / i32::MAX as f32);
                                        }
                                    } else if spec.channels.count() == 2 {
                                        // Stereo - interleave L/R and convert
                                        let left = buf.chan(0);
                                        let right = buf.chan(1);
                                        
                                        for i in 0..left.len() {
                                            samples.push(left[i] as f32 / i32::MAX as f32);
                                            samples.push(right[i] as f32 / i32::MAX as f32);
                                        }
                                    }
                                }
                                symphonia::core::audio::AudioBufferRef::S16(buf) => {
                                    let spec = *buf.spec();
                                    if spec.channels.count() == 1 {
                                        // Mono - convert i16 to f32
                                        let chan = buf.chan(0);
                                        for &sample in chan {
                                            samples.push(sample as f32 / i16::MAX as f32);
                                        }
                                    } else if spec.channels.count() == 2 {
                                        // Stereo - interleave L/R and convert
                                        let left = buf.chan(0);
                                        let right = buf.chan(1);
                                        
                                        for i in 0..left.len() {
                                            samples.push(left[i] as f32 / i16::MAX as f32);
                                            samples.push(right[i] as f32 / i16::MAX as f32);
                                        }
                                    }
                                }
                                _ => {
                                    //debug!("Unsupported audio format");
                                }
                            }
                        }
                        Err(_e) => {
                            //debug!("Decode error: {}", e);
                            break;
                        }
                    }
                }
                Err(_) => break,
            }
        }
        
        // Calculate duration in milliseconds
        let num_frames = samples.len() / channels as usize;
        let duration_ms = (num_frames as f64 / sample_rate as f64 * 1000.0) as u64;
        
        //info!("Decoded {} samples, duration: {}ms", samples.len(), duration_ms);
        Ok((samples, sample_rate, channels, duration_ms))
    }
    
    fn setup_stream(&mut self, samples: Vec<f32>, source_sample_rate: u32, source_channels: u16) -> Result<()> {
        // Get the device's supported configurations
        let supported_configs: Vec<_> = self.device.supported_output_configs()?.collect();
        
        // Find a configuration that matches our requirements
        let mut selected_config = None;
        let mut target_sample_rate = self.forced_sample_rate.unwrap_or(source_sample_rate);
        
        // If we have a forced sample rate, prioritize that
        if let Some(forced_rate) = self.forced_sample_rate {
            for config in &supported_configs {
                if config.min_sample_rate().0 <= forced_rate && 
                   config.max_sample_rate().0 >= forced_rate &&
                   config.channels() == source_channels {
                    selected_config = Some(config.with_sample_rate(SampleRate(forced_rate)));
                    target_sample_rate = forced_rate;
                    //info!("Using forced sample rate: {} Hz", forced_rate);
                    break;
                }
            }
            
            // If forced rate doesn't work with desired channels, try with any channel count
            if selected_config.is_none() {
                for config in &supported_configs {
                    if config.min_sample_rate().0 <= forced_rate && 
                       config.max_sample_rate().0 >= forced_rate {
                        selected_config = Some(config.with_sample_rate(SampleRate(forced_rate)));
                        target_sample_rate = forced_rate;
                        warn!("Using forced sample rate {} Hz but with {} channels instead of requested {}", 
                              forced_rate, config.channels(), source_channels);
                        break;
                    }
                }
            }
        }
        
        // If no forced rate or forced rate didn't work, try to find exact match for source
        if selected_config.is_none() {
            for config in &supported_configs {
                if config.min_sample_rate().0 <= source_sample_rate && 
                   config.max_sample_rate().0 >= source_sample_rate &&
                   config.channels() == source_channels {
                    selected_config = Some(config.with_sample_rate(SampleRate(source_sample_rate)));
                    break;
                }
            }
        
            // If no exact match, find a reasonable alternative
            if selected_config.is_none() {
                for config in &supported_configs {
                    if config.channels() == source_channels {
                        // Use the minimum supported sample rate if source is too low
                        if source_sample_rate < config.min_sample_rate().0 {
                            target_sample_rate = config.min_sample_rate().0;
                            selected_config = Some(config.with_sample_rate(SampleRate(target_sample_rate)));
                            break;
                        }
                        // Use the maximum supported sample rate if source is too high
                        else if source_sample_rate > config.max_sample_rate().0 {
                            target_sample_rate = config.max_sample_rate().0;
                            selected_config = Some(config.with_sample_rate(SampleRate(target_sample_rate)));
                            break;
                        }
                    }
                }
            }
        }
        
        // Fallback to device default if nothing works
        let config = if let Some(config) = selected_config {
            config
        } else {
            warn!("No suitable config found, using device default");
            self.device.default_output_config()?
        };
        
        //info!("Selected config: {} channels, {} Hz, format: {:?}", 
//config.channels(), config.sample_rate().0, config.sample_format());
              
        // Resample if necessary
        let final_samples = if target_sample_rate != source_sample_rate {
            warn!("Resampling from {} Hz to {} Hz", source_sample_rate, target_sample_rate);
            self.resample(samples, source_sample_rate, target_sample_rate, source_channels)
        } else {
            samples
        };
        
        let audio_buffer = Arc::new(Mutex::new(StreamBuffer {
            samples: final_samples,
            position: 0,
            sample_rate: target_sample_rate,
            channels: source_channels,
            volume: self.volume,
            paused: false,
        }));
        
        self.stream_buffer = Some(audio_buffer.clone());
        
        let stream_config = config.config();
        
        // Build stream based on the selected format
        match config.sample_format() {
            SampleFormat::F32 => {
                self.build_f32_stream(stream_config, audio_buffer)?;
            }
            SampleFormat::I32 => {
                self.build_i32_stream(stream_config, audio_buffer)?;
            }
            SampleFormat::I16 => {
                self.build_i16_stream(stream_config, audio_buffer)?;
            }
            _ => {
                return Err(anyhow::anyhow!("Unsupported sample format"));
            }
        }
        
        Ok(())
    }
    
    // Simple linear interpolation resampling (not high quality, but functional)
    fn resample(&self, input: Vec<f32>, input_rate: u32, output_rate: u32, channels: u16) -> Vec<f32> {
        let ratio = input_rate as f64 / output_rate as f64;
        let input_frames = input.len() / channels as usize;
        let output_frames = (input_frames as f64 / ratio) as usize;
        let mut output = Vec::with_capacity(output_frames * channels as usize);
        
        for frame in 0..output_frames {
            let input_frame = (frame as f64 * ratio) as usize;
            
            if input_frame < input_frames {
                for ch in 0..channels {
                    let input_idx = input_frame * channels as usize + ch as usize;
                    if input_idx < input.len() {
                        output.push(input[input_idx]);
                    } else {
                        output.push(0.0);
                    }
                }
            } else {
                for _ in 0..channels {
                    output.push(0.0);
                }
            }
        }
        
        output
    }
    
    fn build_f32_stream(&mut self, config: StreamConfig, audio_buffer: Arc<Mutex<StreamBuffer>>) -> Result<()> {
        let stream = self.device.build_output_stream(
            &config,
            move |data: &mut [f32], _: &cpal::OutputCallbackInfo| {
                let mut buffer = audio_buffer.lock().unwrap();
                
                // If paused, just output silence
                if buffer.paused {
                    data.fill(0.0);
                    return;
                }
                
                let samples_to_copy = data.len().min(buffer.samples.len() - buffer.position);
                
                if samples_to_copy > 0 {
                    let end_pos = buffer.position + samples_to_copy;
                    
                    // Apply volume
                    if buffer.volume < 0.999 {
                        for i in 0..samples_to_copy {
                            data[i] = buffer.samples[buffer.position + i] * buffer.volume;
                        }
                    } else {
                        data[..samples_to_copy].copy_from_slice(&buffer.samples[buffer.position..end_pos]);
                    }
                    
                    buffer.position = end_pos;
                    
                    // Fill remaining with silence if we don't have enough samples
                    if samples_to_copy < data.len() {
                        data[samples_to_copy..].fill(0.0);
                    }
                } else {
                    // No more samples, fill with silence
                    data.fill(0.0);
                }
            },
            |err| error!("ASIO stream error: {}", err),
            None,
        )?;
        
        self.stream = Some(stream);
        Ok(())
    }
    
    fn build_i32_stream(&mut self, config: StreamConfig, audio_buffer: Arc<Mutex<StreamBuffer>>) -> Result<()> {
        let stream = self.device.build_output_stream(
            &config,
            move |data: &mut [i32], _: &cpal::OutputCallbackInfo| {
                let mut buffer = audio_buffer.lock().unwrap();
                
                // If paused, just output silence
                if buffer.paused {
                    data.fill(0);
                    return;
                }
                
                let samples_to_copy = data.len().min(buffer.samples.len() - buffer.position);
                
                if samples_to_copy > 0 {
                    for i in 0..samples_to_copy {
                        data[i] = (buffer.samples[buffer.position + i] * buffer.volume * i32::MAX as f32) as i32;
                    }
                    buffer.position += samples_to_copy;
                    
                    if samples_to_copy < data.len() {
                        data[samples_to_copy..].fill(0);
                    }
                } else {
                    data.fill(0);
                }
            },
            |err| error!("ASIO stream error: {}", err),
            None,
        )?;
        
        self.stream = Some(stream);
        Ok(())
    }
    
    fn build_i16_stream(&mut self, config: StreamConfig, audio_buffer: Arc<Mutex<StreamBuffer>>) -> Result<()> {
        let stream = self.device.build_output_stream(
            &config,
            move |data: &mut [i16], _: &cpal::OutputCallbackInfo| {
                let mut buffer = audio_buffer.lock().unwrap();
                
                // If paused, just output silence
                if buffer.paused {
                    data.fill(0);
                    return;
                }
                
                let samples_to_copy = data.len().min(buffer.samples.len() - buffer.position);
                
                if samples_to_copy > 0 {
                    for i in 0..samples_to_copy {
                        data[i] = (buffer.samples[buffer.position + i] * buffer.volume * i16::MAX as f32) as i16;
                    }
                    buffer.position += samples_to_copy;
                    
                    if samples_to_copy < data.len() {
                        data[samples_to_copy..].fill(0);
                    }
                } else {
                    data.fill(0);
                }
            },
            |err| error!("ASIO stream error: {}", err),
            None,
        )?;
        
        self.stream = Some(stream);
        Ok(())
    }
}