use anyhow::Result;
use serde::{Deserialize, Serialize};
use std::fs;
use std::path::PathBuf;
use tracing::info;

#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct Config {
    pub server_url: String,
    pub user_id: Option<String>,
    pub access_token: Option<String>,
    pub username: Option<String>,
    pub device_id: String,
}

impl Default for Config {
    fn default() -> Self {
        Self {
            server_url: String::new(),
            user_id: None,
            access_token: None,
            username: None,
            device_id: "jellyfin-asio-001".to_string(),
        }
    }
}

impl Config {
    pub fn load() -> Result<Self> {
        let config_path = get_config_path()?;
        
        if !config_path.exists() {
            info!("No config file found, creating default config");
            let default_config = Config::default();
            default_config.save()?;
            return Ok(default_config);
        }
        
        let config_str = fs::read_to_string(&config_path)?;
        let config: Config = serde_json::from_str(&config_str)?;
        
        info!("Loaded config from {}", config_path.display());
        Ok(config)
    }
    
    pub fn save(&self) -> Result<()> {
        let config_path = get_config_path()?;
        
        if let Some(parent) = config_path.parent() {
            fs::create_dir_all(parent)?;
        }
        
        let config_str = serde_json::to_string_pretty(self)?;
        fs::write(&config_path, config_str)?;
        
        info!("Saved config to {}", config_path.display());
        Ok(())
    }
    
    pub fn is_authenticated(&self) -> bool {
        self.user_id.is_some() && self.access_token.is_some()
    }
    
    pub fn clear_auth(&mut self) -> Result<()> {
        self.user_id = None;
        self.access_token = None;
        self.save()
    }
}

fn get_config_path() -> Result<PathBuf> {
    let mut path = dirs::config_dir()
        .ok_or_else(|| anyhow::anyhow!("Could not determine config directory"))?;
    
    path.push("jellyfin-asio-client");
    path.push("config.json");
    
    Ok(path)
}