use anyhow::Result;
use reqwest::Client;
use serde::Deserialize;
use std::collections::HashMap;
use tracing::{info, debug};

use crate::config::Config;

#[derive(Debug, Clone)]
pub struct JellyfinClient {
    client: Client,
    config: Config,
}

#[derive(Debug, Deserialize)]
pub struct AuthResponse {
    #[serde(rename = "User")]
    pub user: User,
    #[serde(rename = "AccessToken")]
    pub access_token: String,
}

#[derive(Debug, Deserialize)]
pub struct User {
    #[serde(rename = "Id")]
    pub id: String,
    #[serde(rename = "Name")]
    pub name: String,
}

#[derive(Debug, Deserialize, Clone)]
pub struct LibraryItem {
    #[serde(rename = "Id")]
    pub id: String,
    #[serde(rename = "Name")]
    pub name: String,
    #[serde(rename = "Type")]
    pub item_type: String,
    #[serde(rename = "Artists")]
    pub artists: Option<Vec<String>>,
    #[serde(rename = "Album")]
    pub album: Option<String>,
    #[serde(rename = "RunTimeTicks")]
    pub runtime_ticks: Option<u64>,
}

#[derive(Debug, Deserialize)]
pub struct ItemsResponse {
    #[serde(rename = "Items")]
    pub items: Vec<LibraryItem>,
}

impl JellyfinClient {
    pub fn new(config: Config) -> Self {
        let client = Client::builder()
            .user_agent("JellyfinASIOClient/0.1.0")
            .build()
            .expect("Failed to create HTTP client");
            
        Self {
            client,
            config,
        }
    }
    
    pub async fn authenticate(&mut self, username: &str, password: &str) -> Result<()> {
        let auth_url = format!("{}/Users/authenticatebyname", self.config.server_url);
        
        let mut auth_data = HashMap::new();
        auth_data.insert("Username", username);
        auth_data.insert("Pw", password);
        
        info!("Authenticating with Jellyfin server...");
        
        let response = self.client
            .post(&auth_url)
            .header("X-Emby-Authorization", format!(r#"MediaBrowser Client="JellyfinASIOClient", Device="Rust Client", DeviceId="{}", Version="0.1.0""#, self.config.device_id))
            .json(&auth_data)
            .send()
            .await?;
            
        if !response.status().is_success() {
            let status = response.status();
            let text = response.text().await?;
            return Err(anyhow::anyhow!("Authentication failed: {} - {}", status, text));
        }
        
        let auth_response: AuthResponse = response.json().await?;
        
        self.config.user_id = Some(auth_response.user.id.clone());
        self.config.access_token = Some(auth_response.access_token.clone());
        self.config.username = Some(username.to_string());
        self.config.save()?;
        
        info!("Authenticated as: {}", auth_response.user.name);
        Ok(())
    }
    
    pub async fn get_music_library(&self) -> Result<Vec<LibraryItem>> {
        let user_id = self.config.user_id.as_ref().ok_or_else(|| anyhow::anyhow!("Not authenticated"))?;
        let token = self.config.access_token.as_ref().ok_or_else(|| anyhow::anyhow!("Not authenticated"))?;
        
        let url = format!("{}/Users/{}/Items", self.config.server_url, user_id);
        
        debug!("Fetching music library...");
        
        let response = self.client
            .get(&url)
            .header("X-Emby-Authorization", format!(r#"MediaBrowser Client="JellyfinASIOClient", Device="Rust Client", DeviceId="{}", Version="0.1.0", Token="{}""#, self.config.device_id, token))
            .query(&[
                ("IncludeItemTypes", "Audio,MusicAlbum"),
                ("Recursive", "true"),
                ("Fields", "Artists,Album,RunTimeTicks"),
                ("SortBy", "Artist,Album,SortName"),
            ])
            .send()
            .await?;
            
        if !response.status().is_success() {
            let status = response.status();
            let text = response.text().await?;
            return Err(anyhow::anyhow!("Failed to fetch library: {} - {}", status, text));
        }
        
        let items_response: ItemsResponse = response.json().await?;
        
        info!("Found {} music items", items_response.items.len());
        Ok(items_response.items)
    }
    
    pub fn get_stream_url(&self, item_id: &str) -> Result<String> {
        let user_id = self.config.user_id.as_ref().ok_or_else(|| anyhow::anyhow!("Not authenticated"))?;
    let token = self.config.access_token.as_ref().ok_or_else(|| anyhow::anyhow!("Not authenticated"))?;
    
    let url = format!(
        "{}/Audio/{}/stream?UserId={}&api_key={}&Static=true",
        self.config.server_url, item_id, user_id, token
    );
        
    Ok(url)
}
    
    pub fn is_authenticated(&self) -> bool {
        self.config.is_authenticated()
    }
    

}

impl LibraryItem {
    pub fn artist_name(&self) -> Option<String> {
        self.artists.as_ref()
            .and_then(|a| a.first())
            .map(|s| s.to_string())
    }
    
    pub fn album_name(&self) -> Option<String> {
        self.album.clone()
    }
}