use anyhow::Result;
use tracing::info;
use std::io::Write;

mod jellyfin;
mod audio;
mod config;
mod player;
mod tui;

#[tokio::main]
async fn main() -> Result<()> {
    tracing_subscriber::fmt::init();
    
    info!("Starting Jellyfin ASIO Client");
    
    let mut config = config::Config::load()?;
    
    // Create Jellyfin client
    let mut jellyfin = jellyfin::JellyfinClient::new(config.clone());
    
    if !jellyfin.is_authenticated() {
        println!("No saved session found. Please log in.");
        
        if config.server_url.is_empty() {
            print!("Enter Jellyfin server URL (e.g., https://jellyfin.example.com): ");
            std::io::stdout().flush()?;
            let mut url = String::new();
            std::io::stdin().read_line(&mut url)?;
            config.server_url = url.trim().to_string();
            config.save()?;
            jellyfin = jellyfin::JellyfinClient::new(config.clone());
        }
        
        print!("Enter your Jellyfin username: ");
        std::io::stdout().flush()?;
        let mut username = String::new();
        std::io::stdin().read_line(&mut username)?;
        username = username.trim().to_string();
        
        print!("Enter your Jellyfin password: ");
        std::io::stdout().flush()?;
        let password = rpassword::read_password()?;
        
        jellyfin.authenticate(&username, &password).await?
    } else {
        info!("Using saved session for {}", config.username.unwrap_or_else(|| "Unknown user".to_string()));
    }
    
    let player = player::Player::new(jellyfin.clone())?;
    
    let mut tui = tui::Tui::new(jellyfin, player)?;
    tui.run().await?;
    
    Ok(())
}
