use anyhow::Result;
use std::sync::{Arc, Mutex};
use tokio::sync::mpsc;
use tracing::error;

use crate::audio::AsioPlayer;
use crate::jellyfin::{JellyfinClient, LibraryItem};

#[derive(Debug, Clone, PartialEq)]
pub enum PlaybackState {
    Stopped,
    Playing,
    Paused,
}

#[derive(Debug, Clone)]
pub enum PlayerCommand {
    Play(String), 
    PlayPause,    
    Stop,
    Next,
    Previous,
    Seek(f32),    
    SetVolume(f32),
}

#[derive(Debug, Clone)]
enum AudioCommand {
    Play(String),
    Pause,
    Resume,
    Stop,
    Seek(f32),
    SetVolume(f32),
}

pub struct PlayerState {
    pub current_item: Option<LibraryItem>,
    pub queue: Vec<LibraryItem>,
    pub queue_position: usize,
    pub playback_state: PlaybackState,
    pub volume: f32,
    pub position: f32, // 0.0 - 1.0
}

impl Default for PlayerState {
    fn default() -> Self {
        Self {
            current_item: None,
            queue: Vec::new(),
            queue_position: 0,
            playback_state: PlaybackState::Stopped,
            volume: 1.0,
            position: 0.0,
        }
    }
}

pub struct Player {
    state: Arc<Mutex<PlayerState>>,
    command_tx: mpsc::Sender<PlayerCommand>,
    audio_tx: mpsc::Sender<AudioCommand>,
    jellyfin: Arc<Mutex<JellyfinClient>>,
}

impl Player {
    pub fn new(jellyfin: JellyfinClient) -> Result<Self> {
        let (command_tx, mut command_rx) = mpsc::channel::<PlayerCommand>(32);
        let (audio_tx, mut audio_rx) = mpsc::channel::<AudioCommand>(32);
        let state = Arc::new(Mutex::new(PlayerState::default()));
        let jellyfin = Arc::new(Mutex::new(jellyfin));
        
        let player_state = state.clone();
        let player_jellyfin = jellyfin.clone();
        let command_tx_clone = command_tx.clone();
        let audio_tx_clone = audio_tx.clone();
        
        // Spawn audio handler in a blocking thread
        let audio_state = state.clone();
        std::thread::spawn(move || {
            let rt = tokio::runtime::Runtime::new().unwrap();
            rt.block_on(async {
                let mut asio_player = match AsioPlayer::new() {
                    Ok(player) => Some(player),
                    Err(e) => {
                        error!("Failed to initialize ASIO player: {}", e);
                        None
                    }
                };
                
                while let Some(cmd) = audio_rx.recv().await {
                    if let Some(ref mut player) = asio_player {
                        match cmd {
                            AudioCommand::Play(url) => {
                                {
                                    let mut state = audio_state.lock().unwrap();
                                    state.playback_state = PlaybackState::Playing;
                                }
                                
                                match player.play_url(&url).await {
                                    Ok(_) => {
                                        let mut state = audio_state.lock().unwrap();
                                        state.playback_state = PlaybackState::Stopped;
                                        state.position = 0.0;
                                    },
                                    Err(e) => error!("Playback error: {}", e)
                                }
                            },
                            AudioCommand::Pause => {
                                if player.pause().is_ok() {
                                    let mut state = audio_state.lock().unwrap();
                                    state.playback_state = PlaybackState::Paused;
                                }
                            },
                            AudioCommand::Resume => {
                                if player.resume().is_ok() {
                                    let mut state = audio_state.lock().unwrap();
                                    state.playback_state = PlaybackState::Playing;
                                }
                            },
                            AudioCommand::Stop => {
                                if player.stop().is_ok() {
                                    let mut state = audio_state.lock().unwrap();
                                    state.playback_state = PlaybackState::Stopped;
                                    state.position = 0.0;
                                }
                            },
                            AudioCommand::Seek(position) => {
                                if player.seek(position).is_ok() {
                                    let mut state = audio_state.lock().unwrap();
                                    state.position = position;
                                }
                            },
                            AudioCommand::SetVolume(volume) => {
                                if player.set_volume(volume).is_ok() {
                                    let mut state = audio_state.lock().unwrap();
                                    state.volume = volume;
                                }
                            }
                        }
                    }
                }
            });
        });
        
        // Spawn command handler task
        tokio::spawn(async move {
            while let Some(cmd) = command_rx.recv().await {
                match cmd {
                    PlayerCommand::Play(item_id) => {
                        // Get the URL
                        let url = {
                            let jellyfin_guard = player_jellyfin.lock().unwrap();
                            jellyfin_guard.get_stream_url(&item_id)
                        };
                        
                        match url {
                            Ok(url) => {
                                let _ = audio_tx_clone.send(AudioCommand::Play(url)).await;
                            },
                            Err(e) => error!("Failed to get stream URL: {}", e)
                        }
                    },
                    PlayerCommand::PlayPause => {
                        // Get the current state
                        let (current_state, item_id) = {
                            let state = player_state.lock().unwrap();
                            (state.playback_state.clone(), state.current_item.as_ref().map(|item| item.id.clone()))
                        };
                        
                        match current_state {
                            PlaybackState::Playing => {
                                let _ = audio_tx_clone.send(AudioCommand::Pause).await;
                            },
                            PlaybackState::Paused => {
                                let _ = audio_tx_clone.send(AudioCommand::Resume).await;
                            },
                            PlaybackState::Stopped => {
                                if let Some(item_id) = item_id {
                                    let _ = command_tx_clone.send(PlayerCommand::Play(item_id)).await;
                                }
                            }
                        }
                    },
                    PlayerCommand::Stop => {
                        let _ = audio_tx_clone.send(AudioCommand::Stop).await;
                    },
                    PlayerCommand::Next => {
                        let next_item_id = {
                            let mut state = player_state.lock().unwrap();
                            if !state.queue.is_empty() && state.queue_position < state.queue.len() - 1 {
                                state.queue_position += 1;
                                Some(state.queue[state.queue_position].id.clone())
                            } else {
                                None
                            }
                        };
                        
                        if let Some(item_id) = next_item_id {
                            let _ = command_tx_clone.send(PlayerCommand::Play(item_id)).await;
                        }
                    },
                    PlayerCommand::Previous => {
                        let prev_item_id = {
                            let mut state = player_state.lock().unwrap();
                            if !state.queue.is_empty() && state.queue_position > 0 {
                                state.queue_position -= 1;
                                Some(state.queue[state.queue_position].id.clone())
                            } else {
                                None
                            }
                        };
                        
                        if let Some(item_id) = prev_item_id {
                            let _ = command_tx_clone.send(PlayerCommand::Play(item_id)).await;
                        }
                    },
                    PlayerCommand::Seek(position) => {
                        let _ = audio_tx_clone.send(AudioCommand::Seek(position)).await;
                    },
                    PlayerCommand::SetVolume(volume) => {
                        let _ = audio_tx_clone.send(AudioCommand::SetVolume(volume)).await;
                    }
                }
            }
        });
        
        Ok(Self {
            state,
            command_tx,
            audio_tx,
            jellyfin,
        })
    }
    
    pub async fn send_command(&self, command: PlayerCommand) -> Result<()> {
        self.command_tx.send(command).await
            .map_err(|e| anyhow::anyhow!("Failed to send player command: {}", e))
    }
    
    pub fn get_state(&self) -> Arc<Mutex<PlayerState>> {
        self.state.clone()
    }
    
    pub fn set_queue(&self, items: Vec<LibraryItem>) -> Result<()> {
        let mut state = self.state.lock().unwrap();
        state.queue = items;
        state.queue_position = 0;
        Ok(())
    }
    
    pub fn get_jellyfin(&self) -> Arc<Mutex<JellyfinClient>> {
        self.jellyfin.clone()
    }
    
    pub fn get_volume(&self) -> f32 {
        let state = self.state.lock().unwrap();
        state.volume
    }
    
    pub fn get_position(&self) -> f32 {
        let state = self.state.lock().unwrap();
        state.position
    }
    
    pub fn get_current_item(&self) -> Option<LibraryItem> {
        let state = self.state.lock().unwrap();
        state.current_item.clone()
    }
    
    pub fn get_playback_state(&self) -> PlaybackState {
        let state = self.state.lock().unwrap();
        state.playback_state.clone()
    }
}